#include "kalman.h"

// Elevator acceleration sensing video switcher
// We were able to get an update every 5 milliseconds,
// Even with floating point math and print()ing data


#define THRESH 15

double x,y,z;
double x_float;
double x_norm, y_norm, z_norm;
double x_norm_temp;
double x_int, y_int, z_int;
double x_avg[100], y_avg[100], z_avg[100];
int x_track, y_track, z_track;
int ctr;
uint32_t last_time = 0;
double time_delta = 0.006;
int x_accum;
uint8_t elev_status;

uint32_t B21_timeout;
uint32_t B12_timeout;

/*
  statuses:
  0: At first floor
  1: Trigger UP
  2: Between First and Second Floors
  3: At second floor
  4: Trigger DN
  5: Between 2nd and first floors
  


*/

uint32_t timeout = 5000;

void setup()
{
  analogReference(EXTERNAL);
  pinMode(2,OUTPUT);
  pinMode(3,OUTPUT);
  digitalWrite(12,HIGH);
  digitalWrite(11,HIGH);

  Serial.begin(9600);
  
Serial.write(0xFE);
Serial.write(0x01);
Serial.write(0x7C);
Serial.write(157);

 // state_x = kalman_init(0.0125,0.01,0.05,-1);
state_x = kalman_init(0.0125,0.1,0.05,-1);

  for(ctr = 0; ctr < 100; ctr++)
  {
    x = float(analogRead(2))*1.5/1024;

    kalman_update(&state_x, x);
    x_avg[ctr] = state_x.x;
  }

  x_norm_temp = 0;
  for(ctr = 0; ctr < 100; ctr++)
  { 
    x_norm_temp += x_avg[ctr];
  }  

  x_norm = x_norm_temp / 100;

}

void loop()
{

  x = (float(analogRead(2))*1.5)/512;

  kalman_update(&state_x, x);

/*
  Serial.print(millis());
  Serial.print(',');
  Serial.print(x,6);
  Serial.print(',');
  Serial.print(state_x.x,6);
*/
  if(x_track < 99)
  {
    x_track++;
    x_avg[x_track] = state_x.x;
  }
  else
  {
    x_track = 0;
    x_avg[x_track] = state_x.x;
  }

  if(millis() > timeout && millis() < timeout+100)
  {
    x_int = 0; 
    elev_status = 0;
  }

  x_norm_temp = 0;
  for(ctr = 0; ctr < 100; ctr++)
  { 
    x_norm_temp += x_avg[ctr];
  }  

  x_norm = x_norm_temp / 100;


  x_int += state_x.x - x_norm;
  
  if(x_accum < 10 && x_int > 0.12)
  {
     x_accum++; // "UP"
  }
  if(x_accum >= 10 && x_accum < THRESH && x_int > 0.20)
  {
     x_accum++; 
  }
  if(x_accum > 0 && x_int < 0.12)
  {
     x_accum--;
  }
  if(x_accum > -10 && x_int < -0.12)
  {
     x_accum--; // "DN"
  }
  if(x_accum <= -10 && x_accum > -THRESH && x_int < -0.20)
  {
     x_accum--; 
  }
  if(x_accum < 0 && x_int > -0.12)
  {
     x_accum++; 
  }

  if(elev_status == 0 && x_accum == -THRESH)
  {
     elev_status = 1; // Going up
  }
  if(elev_status == 1)
  {
    // Trigger UP here
    elev_status = 2;
    B12_timeout = millis() + 11000;
    digitalWrite(2,HIGH);
    delay(100);
    digitalWrite(2,LOW); 
  }
  if(elev_status == 2 && x_accum == THRESH)
  {
    elev_status = 3; // Stopping up
    B12_timeout = 0;
  }
  if(elev_status == 3 && x_accum == 0)
  {
    elev_status = 4; // Stopped up
  }
  if(elev_status == 4 && x_accum == THRESH)
  {
     elev_status = 5; 
  }
  if(elev_status == 5)
  {
     // Trigger down here
     elev_status = 6;
     B21_timeout = millis() + 11000;
    digitalWrite(3,HIGH);
    delay(100);
    digitalWrite(3,LOW); 
  }
  if(elev_status == 6 && x_accum == -THRESH)
  {
     elev_status = 7; // Stopping down 
     B21_timeout = 0;
  }
  if(elev_status == 7 && x_accum == 0)
  {
     elev_status = 0; 
  }
  
  if(B12_timeout != 0 && B12_timeout < millis())
{
  // False up start
  B12_timeout = 0;
  elev_status = 0;
  x_int = 0;

}  

  if(B21_timeout != 0 && B21_timeout < millis())
  {
    // False down start
    B21_timeout = 0;
    elev_status = 4;
    x_int = 0;
  }
     
  //Serial.print(',');
  //Serial.print(x_norm,6);  
/*
  Serial.print(',');
  Serial.println(x_int,6);  
*/

Serial.write(0xFE);
Serial.write(0x80);
Serial.print("V: ");
Serial.print(x_int,6);
Serial.write(0xFE);
Serial.write(0x80+64);
Serial.print("T: ");
Serial.print(x_accum);

Serial.print("   S: ");
switch(elev_status)
{
   case 0:
    Serial.print("@1      ");
   break; 
   case 1:
    Serial.print("UP      ");
   break; 
   case 2:
    Serial.print("B12     ");
   break; 
   case 3:
    Serial.print("S2      ");
   break; 
   case 4:
    Serial.print("@2      ");
   break; 
   case 5:
    Serial.print("DN      ");
   break; 
   case 6:
    Serial.print("B21     ");
   break; 
   case 7:
    Serial.print("S1      ");
   break; 
  
  
}

// Potentially adding buttons to "reset" position
  if(digitalRead(12) == LOW)
  {
    elev_status = 0;
    x_int = 0;
  }
  if(digitalRead(11) == LOW)
  {
    elev_status = 4;
    x_int = 0;
  }  
}

